//
// EngineHandler.cs
//
// Defines an abstract class that the PlayEngineHandler and MenuEngineHandler
//   classes can implement.  The EngineHandler class will call the methods listed
//   here as appropriate.
//

#region Using Directives
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace EfromTibet {
  public abstract class EngineHandler {
    public EngineHandler() {  }

    public abstract bool Initialize();              // called upon XNA Initialize() signal
    public abstract bool LoadContent();             // called upon XNA LoadContent() signal
    public abstract bool UnloadContent();           // called upon XNA UnloadContent() signal
    public abstract void Update(GameTime gameTime); // called when this is the active handler and XNA signals Update()
    public abstract void Draw(GameTime gameTime);   // called when this is the active handler and XNA signals Draw()
    
    public abstract void Activate();                // called when we have switched to this being the active handler
    public abstract void Deactivate();              // called when we have switched from this to a different active handler 
  }
}
