//
// Keyboardandler.cs
//
// This class is used by ControllerEngine and the other EngineHandler subclasses
//   to get keyboard input from the user.  This class acts as a proxy object to the
//   standard keyboard input object included with XNA.  We include this object so that
//   we can add the important WasKeyPressed() method which checks if a key was pressed
//   since the last poll event.
//

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Input;

namespace EfromTibet {
  public class KeyboardHandler {
    protected KeyboardState newState;
    protected KeyboardState oldState;

    public KeyboardHandler(){

    }

    public bool Update() {
      // Update saved state.
      oldState = newState;
      newState = Keyboard.GetState();
      return true;
    }

    public bool IsKeyDown(Keys k) {
      return newState.IsKeyDown(k);
    }

    public bool IsKeyUp(Keys k) {
      return newState.IsKeyUp(k);
    }

    public bool WasKeyPressed(Keys k) {
      // Is the key down currently?
      if (newState.IsKeyDown(k)) {
        // If was not down last update, key has just been pressed.
        if (!oldState.IsKeyDown(k)) {
          return true;
        }
      }
      return false;
    }
  }
}
