using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace EfromTibet {
  public class MenuEngineHandler : EngineHandler {
    
    public enum MenuState {
      MainMenu,         // viewing main menu
      InstructionsMenu, // viewing instructions screen
      CreditsMenu,      // viewing credits screen
      InputOptionMenu,  // menu prompting which input method to use
      CalibrationWait   // waiting for camera calibration screen completion
    }




    #region Constants
    //protected readonly Color TITLETEXTCOLOR = Color.SandyBrown;
    protected readonly Color UNSELECTEDCHOICECOLOR = Color.Black;
    protected readonly Color SELECTEDCHOICECOLOR = Color.OrangeRed;

    protected const int CREDITS_TOP = 10;
    protected const int CREDITS_LEFT = 10;
    protected const String CREDITS_TEXT = "-----Escape from Tibet Credits-----\n \nProgramming:\n     Aaron Curley\n     Bilal Ghalib\n \nArtwork:\n     Kathy Curley\n     Ruth Curley\n     Christy Deering\n     Aaron Curley\n     Bilal Ghalib\n \nCamera Input:\n     Open Source Computer Vision Library";
    
    protected const int INSTRUCTIONS_TOP = 10;
    protected const int INSTRUCTIONS_LEFT = 10;
    protected const String INSTRUCTIONS_TEXT = "-----Escape from Tibet Instructions-----\ntwolop\nAs James Bond you've gathered important information at Top Secret HQ atop \na mountain in Tibet. The information \nin it is critically time sensitive, you must ski down the slope to your car and phone \nthe Prime Minister as quickly as possible!\n\nAs bond you have many tricks up your sleeve, and thus you have multiple ways \nto control your MechaSki's\ntwolop\n--BY MOUSE--\nBy moving the mouse up and down on the screen you \ncan control the speed. Moving the mouse left and right angles the ski's as you \nspeed down the slopes! Try clicking to jump over objects.\ntwolop\n--BY KEYBOARD--\nUp  Arrow speeds you up. Down Arrow slows you down. \nLeft arrow moves you left, and right arrow moves you right.\n Pressing the space bar let's you jump.\ntwolop\n--BY CAMERA--\nFace recognition recognizes your face. So if you put your face closer you will speed up. Pulling your head backward slows you down. \nBy moving your head left and right the skis will move too. \n Moving your head in quickly makes you jump. ";

    protected readonly Color CREDITS_TEXT_COLOR = Color.Black;
    #endregion



    
    #region Variable Defs
    protected MenuState m_menuState;
    protected Texture2D m_mainMenuBackground;
    protected Rectangle m_mainMenuBackground_dest;
    protected Texture2D m_mainMenuLight;
    protected Rectangle m_mainMenuLight_dest;
    protected SpriteFont m_mainMenuTitleFont;
    protected SpriteFont m_VersionFont;
    protected SpriteFont m_MenuChoiceFont;
    protected SpriteFont m_SubmenuChoiceFont;

    protected String[] m_mainMenuChoices = {"Start Game", "Instructions", "Credits", "Exit Game"};
    protected Rectangle[] m_mainMenuChoices_rects;

    protected String m_instructionsMenuSingleChoice = "Back to Main Menu";

    protected String[] m_cameraOptionMenuChoices = { "Use Camera input", "Use Mouse Input", "Use Keyboard Input", "Back" };
    protected Rectangle[] m_cameraOptionMenuChoices_rects;

    protected int m_currentChoice;
    protected int m_lastMouseX;
    protected int m_lastMouseY;
    #endregion




    #region Constructor
    public MenuEngineHandler() {

    }
    #endregion




    #region Initialize()
    public override bool Initialize() {
      return true;
    }
    #endregion




    #region LoadContent()
    public override bool LoadContent() {
      try {
        ContentManager c = Program.controller.Content;
        
        m_mainMenuBackground = c.Load<Texture2D>("TitleScreenBackground");
        m_mainMenuBackground_dest = new Rectangle(0, 0, Program.controller.windowSize.Width, Program.controller.windowSize.Height);

        m_mainMenuLight = c.Load<Texture2D>("TitleScreenLight");
        m_mainMenuLight_dest = new Rectangle(0, 0, Program.controller.windowSize.Width, Program.controller.windowSize.Height);

        m_mainMenuTitleFont = c.Load<SpriteFont>("TitleText");

        m_VersionFont = c.Load<SpriteFont>("VersionText");

        m_MenuChoiceFont = c.Load<SpriteFont>("MenuChoiceText");

        m_SubmenuChoiceFont = c.Load<SpriteFont>("SubMenuChoiceText");

        return true;
      } catch {
        return false;
      }
    }
    #endregion




    #region UnloadContent()
    public override bool UnloadContent() {
      return true;
    }
    #endregion




    #region Update()
    public override void Update(GameTime gameTime) {
      ControllerEngine controller = Program.controller;
      
      bool upKeyAction = false;
      bool downKeyAction = false;
      bool executeAction = false;
        
      // translate key presses to desired actions
      if ((controller.keyboard.WasKeyPressed(Keys.Up)) || (controller.keyboard.WasKeyPressed(Keys.W))) {
        upKeyAction = true;
      }
      if ((controller.keyboard.WasKeyPressed(Keys.Down)) || (controller.keyboard.WasKeyPressed(Keys.S))) {
        downKeyAction = true;
      }


      // check if user has pressed ENTER, SPACE, or clicked with the mouse
      // if so, we should execute the current action
      if ((controller.keyboard.WasKeyPressed(Keys.Enter)) ||
          (controller.keyboard.WasKeyPressed(Keys.Space)) ||
          (controller.mouse.WasLeftButtonPressed())) {
        if ((controller.keyboard.IsKeyDown(Keys.LeftAlt) == false) &&
            (controller.keyboard.IsKeyDown(Keys.RightAlt) == false)) {
          executeAction = true;
        }
      }




      // ********************************************************************
      // Code for when currently viewing the MAIN MENU.
      if (m_menuState == MenuState.MainMenu){
        
        // adjusts our number of course objects
        if (controller.keyboard.WasKeyPressed(Keys.OemPlus)) {
          Program.controller.playEngine.numStaticActorsToGenerate += PlayEngineHandler.STATIC_ACTORS_TO_GENERATE_STEP;
          Program.controller.playEngine.regenerateCourse();
        }else if (controller.keyboard.WasKeyPressed(Keys.OemMinus)) {
          Program.controller.playEngine.numStaticActorsToGenerate -= PlayEngineHandler.STATIC_ACTORS_TO_GENERATE_STEP;
          if (Program.controller.playEngine.numStaticActorsToGenerate < 0) {
            Program.controller.playEngine.numStaticActorsToGenerate = 0;
          }
          Program.controller.playEngine.regenerateCourse();
        }


        
        
        if (m_currentChoice == -1) {
          m_currentChoice = 0;
          m_lastMouseX = -1;
          m_lastMouseY = -1;
        }

        // take actions of key presses
        if (upKeyAction){
          m_currentChoice--;
          if (m_currentChoice < 0) m_currentChoice = m_mainMenuChoices.Length-1;
        }else if(downKeyAction){
          m_currentChoice++;
          if (m_currentChoice > m_mainMenuChoices.Length-1) m_currentChoice = 0;
        }

        // if mouse has been moved
        if ((controller.mouse.Y != m_lastMouseY) || (controller.mouse.X != m_lastMouseX)) {
          int mouseChoice = -1;

          // check if mouse is over any of the choices
          if (m_mainMenuChoices_rects != null) {
            for (int i = 0; i < m_mainMenuChoices_rects.Length; i++) {
              if (
                  ((controller.mouse.Y >= m_mainMenuChoices_rects[i].Y) && (controller.mouse.Y < m_mainMenuChoices_rects[i].Bottom)) &&
                  ((controller.mouse.X >= m_mainMenuChoices_rects[i].X) && (controller.mouse.X < m_mainMenuChoices_rects[i].Right))
                 ) {
                mouseChoice = i;
                break;
              }
            }
          }

          // if the mouse is over one of the choices, set that as our choice
          if ((mouseChoice >= 0) && (mouseChoice <= m_mainMenuChoices.Length-1)) {
            m_currentChoice = mouseChoice;
          }
        }
        
        // if the user executed the current action
        if (executeAction) {
          switch (m_currentChoice) {
            // start game
            case 0:
              m_menuState = MenuState.InputOptionMenu;
              
              m_currentChoice = -1;
              break;
            
            // instructions
            case 1:
              m_menuState = MenuState.InstructionsMenu;
              m_currentChoice = -1;
              break;

            // credits
            case 2:
              m_menuState = MenuState.CreditsMenu;
              m_currentChoice = -1;
              break;

            // exit game
            case 3:
              Program.controller.exitGame();
              break;
          }
        }
      }
      // ********************************************************************




      // ********************************************************************
      // Code for when viewing the Instructions Menu or Credits Menu
      else if((m_menuState == MenuState.InstructionsMenu) || 
              (m_menuState == MenuState.CreditsMenu)      ){
        m_currentChoice = 0;
        
        // since we have only one choice (back to main menu), we don't need to "check" our selection
        if (executeAction){
          m_menuState = MenuState.MainMenu;
          m_currentChoice = -1;
        }
      }
      // ********************************************************************




      // ********************************************************************
      // Code for when currently viewing the CAMERA OPTIONS MENU.
      else if (m_menuState == MenuState.InputOptionMenu) {
        if (m_currentChoice == -1) {
          m_currentChoice = 0;
          m_lastMouseX = -1;
          m_lastMouseY = -1;
        }

        // take actions of key presses
        if (upKeyAction) {
          m_currentChoice--;
          if (m_currentChoice < 0) m_currentChoice = m_cameraOptionMenuChoices.Length - 1;
        } else if (downKeyAction) {
          m_currentChoice++;
          if (m_currentChoice > m_cameraOptionMenuChoices.Length - 1) m_currentChoice = 0;
        }

        // if mouse has been moved
        if ((controller.mouse.Y != m_lastMouseY) || (controller.mouse.X != m_lastMouseX)) {
          int mouseChoice = -1;

          // check if mouse is over any of the choices
          if (m_cameraOptionMenuChoices_rects != null) {
            for (int i = 0; i < m_cameraOptionMenuChoices_rects.Length; i++) {
              if (
                  ((controller.mouse.Y >= m_cameraOptionMenuChoices_rects[i].Y) && (controller.mouse.Y < m_cameraOptionMenuChoices_rects[i].Bottom)) &&
                  ((controller.mouse.X >= m_cameraOptionMenuChoices_rects[i].X) && (controller.mouse.X < m_cameraOptionMenuChoices_rects[i].Right))
                 ) {
                mouseChoice = i;
                break;
              }
            }
          }

          // if the mouse is over one of the choices, set that as our choice
          if ((mouseChoice >= 0) && (mouseChoice <= m_cameraOptionMenuChoices_rects.Length - 1)) {
            m_currentChoice = mouseChoice;
          }
        }

        // if the user executed the current action
        if (executeAction) {
          switch (m_currentChoice) {
            // use camera input choice
            case 0:
              // go to calibration state
              m_menuState = MenuState.CalibrationWait;
              m_currentChoice = -1;
              break;

            // use mouse input choice
            case 1:
              controller.inputToUse = ControllerEngine.InputType.Mouse;
              controller.switchToPlaying();
              m_currentChoice = -1;
              break;

            // use keyboard input choice
            case 2:
              controller.inputToUse = ControllerEngine.InputType.Keyboard;
              controller.switchToPlaying();
              m_currentChoice = -1;
              break;

            // back
            case 3:
              m_menuState = MenuState.MainMenu;
              m_currentChoice = -1;
              break;
          }
        }
      }
      
      
      
      
      // if currently viewing "Calibration Wait" screen
      else if (m_menuState == MenuState.CalibrationWait) {
        // if first time through
        if (m_currentChoice == -1) {
          controller.inputToUse = ControllerEngine.InputType.Camera;

          if (controller.camera.initialize() == true) {
            
          }
                    
          m_currentChoice = 0;
        } else {
          if (controller.camera.isInitialized()){
            controller.switchToPlaying();
          }
          
          if ((controller.mouse.WasLeftButtonPressed()) || (controller.keyboard.WasKeyPressed(Keys.Enter)) || (controller.keyboard.WasKeyPressed(Keys.Space))) {
            m_menuState = MenuState.MainMenu;
            m_currentChoice = -1;
          }
        }

      }
      // ********************************************************************





      // save the current mouse values as "previous" values for next loop
      m_lastMouseX = controller.mouse.X;
      m_lastMouseY = controller.mouse.Y;
      return;
    }
    #endregion




    #region Draw()
    public override void Draw(GameTime gameTime) {
      GraphicsDeviceManager g = Program.controller.graphics;
      SpriteBatch spriteHandler = Program.controller.spriteHandler;

      // Clear background
      g.GraphicsDevice.Clear(Color.Black);

      // Start rendering sprites
      spriteHandler.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);




        

        



        // if viewing main menu, draw all main-menu related stuff
        if (m_menuState == MenuState.MainMenu) {
          // draw the main menu background image
          spriteHandler.Draw(m_mainMenuBackground, m_mainMenuBackground_dest, Color.White);



          // ************************************
          // draw the menu choices to choose from

          // if we have already calculated the rectangles for the choices, we can just draw them 
          if (m_mainMenuChoices_rects != null) {
            // draw all menu choices, using the previously-calculated rectangles
            for (int i = 0; i < m_mainMenuChoices.Length; i++) {
              // destination is previously-calculated rectangle
              Vector2 dest;
              dest.X = m_mainMenuChoices_rects[i].X;
              dest.Y = m_mainMenuChoices_rects[i].Y;

              // if this is the currently selected choice, use color orange, otherwise maroon
              Color c = UNSELECTEDCHOICECOLOR;
              if (i == m_currentChoice) c = SELECTEDCHOICECOLOR;

              // draw the text
              spriteHandler.DrawString(m_MenuChoiceFont, m_mainMenuChoices[i], dest, c);
            }

          // if we have not already calculated and saved the rectangles for the choices, then we need to calculate them as we draw
          } else {
            // reserve space for rectangles
            m_mainMenuChoices_rects = new Rectangle[m_mainMenuChoices.Length];
            
            // calculate text height
            float choiceTextHeight = m_MenuChoiceFont.MeasureString("TestingHeight").Y;
            
            // calculate the initial Y position of the first menu item.  
            //   currentMenuY = (middle of the window) - (half of the entire block of menu choices)
            float currentMenuY = Program.controller.windowSize.Height / 2 - (float)((choiceTextHeight * 1.5)*m_mainMenuChoices.Length) / 2;         
            // move the choices down just a little bit
            currentMenuY += Program.controller.windowSize.Height / 16;

            // draw all menu choices, calculating and saving rectangles as we go
            for (int i = 0; i < m_mainMenuChoices.Length; i++) {
              // finds the text width
              Vector2 curChoiceTextSize = m_MenuChoiceFont.MeasureString(m_mainMenuChoices[i]);
              
              // position this text in our current Y position and centered horizontally
              Vector2 dest;
              dest.Y = currentMenuY;
              dest.X = Program.controller.windowSize.Width / 2 - curChoiceTextSize.X / 2;

              // if this is the currently selected choice, use color orange, otherwise maroon
              Color c = UNSELECTEDCHOICECOLOR;
              if (i == m_currentChoice) c = SELECTEDCHOICECOLOR;
              
              // draw the text
              spriteHandler.DrawString(m_MenuChoiceFont, m_mainMenuChoices[i], dest, c);

              // save this rectangle so we don't have to calculate it later
              m_mainMenuChoices_rects[i].X = (int)dest.X;
              m_mainMenuChoices_rects[i].Y = (int)dest.Y;
              m_mainMenuChoices_rects[i].Width = (int)curChoiceTextSize.X;
              m_mainMenuChoices_rects[i].Height = (int)curChoiceTextSize.Y;

              // move our "current Y" position down 1.5 rows
              currentMenuY += (float)(choiceTextHeight * 1.5);
            }
          }



          // draw "number of course objects" text
          Vector2 textdest;
          textdest.X = 0;
          textdest.Y = 0;
          String todraw = "Number of objects to generate: " + Program.controller.playEngine.numStaticActorsToGenerate.ToString();
          Vector2 textsize = m_VersionFont.MeasureString(todraw);
          spriteHandler.DrawString(m_VersionFont, todraw, textdest, Color.Red);
          textdest.X = 0;
          textdest.Y = textsize.Y;
          spriteHandler.DrawString(m_VersionFont, "(Use + and - keys to adjust)", textdest, Color.Red);



          // done drawing the menu choices to choose from
          // ************************************
        }




        // if viewing "instructions" menu or "credits" menu
        else if ((m_menuState == MenuState.InstructionsMenu) || (m_menuState == MenuState.CreditsMenu)) {
          // draw the main menu background image
          spriteHandler.Draw(m_mainMenuLight, m_mainMenuLight_dest, Color.White);

          
          Vector2 dest;
          Vector2 textSize;

          // draw "back to main menu" text
          textSize = m_SubmenuChoiceFont.MeasureString(m_instructionsMenuSingleChoice);
          dest.X = Program.controller.windowSize.Width - (float)(textSize.X * 1.0) - (float)(textSize.Y * 1.5);
          dest.Y = Program.controller.windowSize.Height - (float)(textSize.Y * 1.5);
          spriteHandler.DrawString(m_SubmenuChoiceFont, m_instructionsMenuSingleChoice, dest, SELECTEDCHOICECOLOR);


          if (m_menuState == MenuState.CreditsMenu){
            String[] lineStrs = CREDITS_TEXT.Split('\n');
            int currentY = CREDITS_TOP;
            int height;
            for (int i = 0; i < lineStrs.Length; i++){
              dest = m_SubmenuChoiceFont.MeasureString(lineStrs[i]);
              height = (int)dest.Y;
              dest.Y = currentY;
              dest.X = CREDITS_LEFT;
              spriteHandler.DrawString(m_SubmenuChoiceFont, lineStrs[i], dest, CREDITS_TEXT_COLOR);

              currentY += height;
            }
          }

          else if (m_menuState == MenuState.InstructionsMenu)
          {
              String[] lineStrs = INSTRUCTIONS_TEXT.Split('\n');
              int currentY = INSTRUCTIONS_TOP;

              int height;
              for (int i = 0; i < lineStrs.Length; i++)
              {

                  dest = m_SubmenuChoiceFont.MeasureString(lineStrs[i]);
                  height = (int)dest.Y;
                  if (lineStrs[i] == "twolop") // this code checks to see if twolop is inserted into the text, this makes a double \n
                  {
                      currentY += 40;
                      i++;
                  }
                  dest.Y = currentY;
                  dest.X = INSTRUCTIONS_LEFT; 
                  spriteHandler.DrawString(m_SubmenuChoiceFont, lineStrs[i], dest, CREDITS_TEXT_COLOR);
                  if (lineStrs[i] != "twolop")
                  {
                      currentY += height;
                  }
              }
          }

        }
        
        
        
        
        // if viewing camera options menu (enable camera input, disable camera input, back)
        else if (m_menuState == MenuState.InputOptionMenu) {
          // draw the main menu background image
          spriteHandler.Draw(m_mainMenuLight, m_mainMenuLight_dest, Color.White);
          


          // ************************************
          // draw the camera option choices to choose from

          // if we have already calculated the rectangles for the choices, we can just draw them 
          if (m_cameraOptionMenuChoices_rects != null) {
            // draw all menu choices, using the previously-calculated rectangles
            for (int i = 0; i < m_cameraOptionMenuChoices_rects.Length; i++) {
              // destination is previously-calculated rectangle
              Vector2 dest;
              dest.X = m_cameraOptionMenuChoices_rects[i].X;
              dest.Y = m_cameraOptionMenuChoices_rects[i].Y;

              // if this is the currently selected choice, use color orange, otherwise maroon
              Color c = UNSELECTEDCHOICECOLOR;
              if (i == m_currentChoice) c = SELECTEDCHOICECOLOR;

              // draw the text
              spriteHandler.DrawString(m_SubmenuChoiceFont, m_cameraOptionMenuChoices[i], dest, c);
            }

            // if we have not already calculated and saved the rectangles for the choices, then we need to calculate them as we draw
          } else {
            // reserve space for rectangles
            m_cameraOptionMenuChoices_rects = new Rectangle[m_cameraOptionMenuChoices.Length];

            // calculate text height
            float choiceTextHeight = m_SubmenuChoiceFont.MeasureString("TestingHeight").Y;

            // calculate the initial Y position of the first menu item.  
            //   currentMenuY = (middle of the window) - (half of the entire block of menu choices)
            float currentMenuY = Program.controller.windowSize.Height / 2 - (float)((choiceTextHeight * 1.5) * m_cameraOptionMenuChoices.Length) / 2;
            // move the choices down just a little bit
            currentMenuY += Program.controller.windowSize.Height / 16;

            // draw all menu choices, calculating and saving rectangles as we go
            for (int i = 0; i < m_cameraOptionMenuChoices.Length; i++) {
              // finds the text width
              Vector2 curChoiceTextSize = m_SubmenuChoiceFont.MeasureString(m_cameraOptionMenuChoices[i]);

              // position this text in our current Y position and centered horizontally
              Vector2 dest;
              dest.Y = currentMenuY;
              dest.X = Program.controller.windowSize.Width / 2 - curChoiceTextSize.X / 2;

              // if this is the currently selected choice, use color orange, otherwise maroon
              Color c = UNSELECTEDCHOICECOLOR;
              if (i == m_currentChoice) c = SELECTEDCHOICECOLOR;

              // draw the text
              spriteHandler.DrawString(m_SubmenuChoiceFont, m_cameraOptionMenuChoices[i], dest, c);

              // save this rectangle so we don't have to calculate it later
              m_cameraOptionMenuChoices_rects[i].X = (int)dest.X;
              m_cameraOptionMenuChoices_rects[i].Y = (int)dest.Y;
              m_cameraOptionMenuChoices_rects[i].Width = (int)curChoiceTextSize.X;
              m_cameraOptionMenuChoices_rects[i].Height = (int)curChoiceTextSize.Y;

              // move our "current Y" position down 1.5 rows
              currentMenuY += (float)(choiceTextHeight * 1.5);
            }
          }
          // done drawing the menu choices to choose from
          // ************************************
        }
        
        
        
        
        // if currently "calibrating" camera
        else if (m_menuState == MenuState.CalibrationWait) {
          // draw the main menu background image
          spriteHandler.Draw(m_mainMenuLight, m_mainMenuLight_dest, Color.White);
          
          
          // display "calibrating camera" text
          Vector2 size = m_MenuChoiceFont.MeasureString("Calibrating Camera...");
          Vector2 dest;

          dest.X = 0;
          dest.Y = 0;
          spriteHandler.DrawString(m_MenuChoiceFont, "Calibrating Camera..." , dest, Color.Red);

          dest.X = 0;
          dest.Y = Program.controller.windowSize.Height - size.Y;
          spriteHandler.DrawString(m_MenuChoiceFont, "Calibrating Camera...", dest, Color.Red);

          dest.X = Program.controller.windowSize.Width - size.X;
          dest.Y = Program.controller.windowSize.Height - size.Y;
          spriteHandler.DrawString(m_MenuChoiceFont, "Calibrating Camera...", dest, Color.Red);

          dest.X = Program.controller.windowSize.Width - size.X;
          dest.Y = 0;
          spriteHandler.DrawString(m_MenuChoiceFont, "Calibrating Camera...", dest, Color.Red);


          if (m_currentChoice >= 0) {
            String toDisplay1 = "We're sorry, your camera could not be initialized.";
            String toDisplay2 = "Click or press ENTER to return to the main menu.";
            Vector2 size1 = m_SubmenuChoiceFont.MeasureString(toDisplay1);
            Vector2 size2 = m_SubmenuChoiceFont.MeasureString(toDisplay2);

            dest.Y = Program.controller.windowSize.Height * 2 / 3;
            dest.X = Program.controller.windowSize.Width/2 - size1.X / 2;
            spriteHandler.DrawString(m_SubmenuChoiceFont, toDisplay1, dest, Color.Red);

            dest.Y = Program.controller.windowSize.Height * 2 / 3 + size1.Y * 2;
            dest.X = Program.controller.windowSize.Width / 2 - size2.X / 2;
            spriteHandler.DrawString(m_SubmenuChoiceFont, toDisplay2, dest, Color.Red);
          }

          //OpenCVProxy c = Program.controller.camera;
          //FaceRecord fRec = c.faceInfo;

          //if ((fRec.x >= 0) && (fRec.y >= 0)) {
            //dest.X = 100;
            //dest.Y = 100;
            //spriteHandler.DrawString(m_SubmenuChoiceFont, "X: " + fRec.x.ToString(), dest, Color.Red);
            //dest.Y = 130;
            //spriteHandler.DrawString(m_SubmenuChoiceFont, "Y: " + fRec.y.ToString(), dest, Color.Red);
            //dest.Y = 160;
            //spriteHandler.DrawString(m_SubmenuChoiceFont, "Height: " + fRec.height.ToString(), dest, Color.Red);
            //dest.Y = 190;
            //spriteHandler.DrawString(m_SubmenuChoiceFont, "Width: " + fRec.width.ToString(), dest, Color.Red);
          //}
        }
        
        // draw the version number text 
        //   text is drawn in upper-right corner
        Vector2 verSize = m_VersionFont.MeasureString(Program.VERSION_STRING);
        Vector2 verPos;
        verPos.X = Program.controller.windowSize.Width - verSize.X;
        verPos.Y = 0;
        spriteHandler.DrawString(m_VersionFont, Program.VERSION_STRING, verPos, Color.Red);


      // We are done, draw everything on screen with help of the end method.
      spriteHandler.End();
    }
    #endregion




    #region Activate()
    public override void Activate() {
      m_menuState = MenuState.MainMenu; // start out on the main menu
      m_currentChoice = 0;
      m_lastMouseX = -1;
      m_lastMouseY = -1;
    }
    #endregion




    #region Deactivate()
    public override void Deactivate() {

    }
    #endregion

  }
}
