//
// MouseHandler.cs
//
// This class is used by ControllerEngine and the other EngineHandler subclasses
//   to get mouse input from the user.  This class acts as a proxy object to the
//   standard mouse input object included with XNA.  We include this object so that
//   we can add the important WasLeftButtonPressed() and WasRightButtonPressed() methods.
//

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Input;

namespace EfromTibet {
  public class MouseHandler {
    protected MouseState newState;
    protected MouseState oldState;

    public MouseHandler(){

    }

    public bool Update() {
      // Update saved state.
      oldState = newState;
      newState = Mouse.GetState();
      return true;
    }

    public int Y {
      get {
        return newState.Y;
      }
    }

    public int X {
      get {
        return newState.X;
      }
    }

    public ButtonState LeftButton{
      get {
        return newState.LeftButton;
      }
    }

    public ButtonState RightButton {
      get {
        return newState.RightButton;
      }
    }

    public ButtonState MiddleButton {
      get {
        return newState.MiddleButton;
      }
    }

    public bool WasLeftButtonPressed() {
      // Is the button down currently?
      if (newState.LeftButton == ButtonState.Pressed) {
        // If was not down last update, button has just been pressed.
        if (oldState.LeftButton != ButtonState.Pressed){
          return Program.controller.IsActive;
        }
      }
      return false;
    }

    public bool WasRightButtonPressed() {
      // Is the button down currently?
      if (newState.RightButton == ButtonState.Pressed) {
        // If was not down last update, button has just been pressed.
        if (oldState.RightButton != ButtonState.Pressed){
          return Program.controller.IsActive;
        }
      }
      return false;
    }
  }
}
