//
// OpenCVProxy.cs
//
// This class creates a simple-to-use interface to the OpenCV libraries.
//

#region Using Directives
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using OpenCVWrapper;

#endregion

namespace EfromTibet
{

    // proxy object interacts with OpenCV.NET
    public struct FaceRecord
    {
        public static  bool currentlyRecognized;
        public static int x;
        public static int y;
        public static int width;
        public static int height;
        public static int camWidth;
        public static int camHeight;
        public static int numpoll;
        public static bool clicked;
    }

    public class OpenCVProxy
    {
        protected OCVWrap m_wrapper = new OCVWrap();
        protected bool m_initSuccess = false;


        protected Size m_cameraSize;
        protected FaceRecord m_record;


        public FaceRecord faceInfo { get { return m_record; } }
        public Size cameraSize { get { return m_cameraSize; } }

        protected int m_failedToLocateCounter = 0;


        int[] faceAreas = new int[5];
        public OpenCVProxy()
        {

        }



        // initializes the camera
        public bool initialize()
        {
            m_initSuccess = m_wrapper.initialize();
            if (m_initSuccess == false) return false;

            FaceRecord.clicked = false;
          //  m_initSuccess = this.poll();
            if (m_initSuccess == false)
            {
                m_wrapper.deinitialize();
                return false;
            }

            m_cameraSize.Width = m_wrapper.getCameraWidth();
            m_cameraSize.Height = m_wrapper.getCameraHeight();

            return true;
        }



        public void deactivate()
        {
            if (m_initSuccess == false) return;
            m_initSuccess = false;
            m_wrapper.deinitialize();
            return;
        }



        public bool isInitialized()
        {
            return m_initSuccess;
        }



        public void poll()
        {
            bool pollSuccess;
            pollSuccess = m_wrapper.pollFaceCoordinates();

            if (pollSuccess == false)
            {
        //        return false;
            }

            FaceRecord.numpoll++;
            int numFaces = m_wrapper.getNumFaces();


            FaceRecord.clicked = false;

            // if we found at least one face,
            if (numFaces > 0)
            {

                // find the "largest" face
                // also check if the user clicks with his face
                int largestArea = 0;
                int largestID = 0;
                int place;
                int topSize = 0, bottomSize = 0;
                place = (int)FaceRecord.numpoll % 5;
                for (int i = 0; i < numFaces; i++)
                {
                    int faceArea = m_wrapper.getWidth(i) * m_wrapper.getHeight(i);
                    if (faceArea >= largestArea)
                    {
                        largestID = i;
                    }
                    faceAreas[place] = faceArea;
                }
                topSize = faceAreas[0];
                bottomSize = faceAreas[4];
                int smallestID = 9999, biggestID = 0, range = 0;
                float mean=0;
                for (int j = 0; j < 5; j++)
                {
                    if (faceAreas[j] > biggestID)
                        biggestID = j;
                    if (faceAreas[j] < smallestID)
                        smallestID = j;
                    mean += faceAreas[j];
                }
                mean=mean/4;
                range = biggestID - smallestID;

                if ((mean>(range*.75))&&(mean>(faceAreas[smallestID]*1.75)))
                    FaceRecord.clicked = true;

                // 
                FaceRecord.x = m_wrapper.getX(largestID);
                FaceRecord.y = m_wrapper.getY(largestID);
                FaceRecord.height = m_wrapper.getHeight(largestID);
                FaceRecord.width = m_wrapper.getWidth(largestID);
                FaceRecord.currentlyRecognized = true;
                FaceRecord.camHeight = m_wrapper.getCameraHeight();
                FaceRecord.camWidth = m_wrapper.getCameraWidth();

                m_failedToLocateCounter = 0;

                // did not find any faces
            }
            else
            {
                m_failedToLocateCounter++;
                FaceRecord.currentlyRecognized = false;

                if (m_failedToLocateCounter > 10)
                {
                    FaceRecord.x = -1;
                    FaceRecord.y = -1;
                    FaceRecord.height = -1;
                    FaceRecord.width = -1;
                    m_failedToLocateCounter = 0;
                }
            }
           // return true;
        }
    }
}
