using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace EfromTibet {
  public class Player : Actor {
    public const int STATE_MOVING = 0;
    public const int STATE_DYING = 1;
    public const int STATE_DEAD = 2;
    public const int STATE_FALLING = 3;
    public const int STATE_GETTINGUP = 4;  // we'll actually use this state for the "stuck in snow animation"

    protected Texture2D m_playerHeadTex;
    protected Texture2D m_playerHeadLeftTex;
    protected Texture2D m_playerHeadExtremeLeftTex;
    protected Texture2D m_playerHeadRightTex;
    protected Texture2D m_playerHeadExtremeRightTex;
    
    protected Texture2D m_playerFallingOne;
    protected Texture2D m_playerFallingTwo;
    protected Texture2D m_currentFallingFrame;  // which of the above two to display

    protected Texture2D m_playerStuckInSnow; 
          
    public const int GETTINGUP_TIME_MILLIS = 1000;
    public const int FLYFORWARD_TIME_MILLIS = 500;
    public const int FALLING_STATE_TIMEOUT = 2500;

    protected int m_stateTimer;

    public override int state {
      get{
        return m_state;
      }
      set {
        m_state = value;
        m_stateTimer = 0;
        if (m_state == STATE_MOVING) {
          m_objectSize.Y = 200;
          m_objectSize.X = m_objectSize.Y * imgSize.Width / imgSize.Height;
          m_objectSize.Z = m_objectSize.X;
          m_collisionSize = m_objectSize;
        } else if ((m_state == STATE_DYING) || (m_state == STATE_FALLING)) {
          m_objectSize.Y = 500;
          m_objectSize.X = m_objectSize.Y;// *m_playerDyingTex.Width / m_playerDyingTex.Height;
          m_objectSize.Z = m_objectSize.X;
          m_currentFallingFrame = m_playerFallingOne;
        }
      }
    }

    public float speed;
    public float vSpeed;

    public float angle;

    public int health;  // health from 0 to 100

    public override bool initialize(Microsoft.Xna.Framework.Content.ContentManager c) {
      try{
        m_playerHeadTex = c.Load<Texture2D>("PlayerHead");
            
        m_playerHeadExtremeLeftTex = c.Load<Texture2D>("PlayerExtremeTiltLeft");
        m_playerHeadExtremeRightTex = c.Load<Texture2D>("PlayerExtremeTiltRight");
        m_playerHeadLeftTex = c.Load<Texture2D>("PlayerTiltLeft");
        m_playerHeadRightTex = c.Load<Texture2D>("PlayerTiltRight");

        m_playerFallingOne = c.Load<Texture2D>("FallingOne");
        m_playerFallingTwo = c.Load<Texture2D>("FallingTwo");
        
        m_playerStuckInSnow = c.Load<Texture2D>("FallingThree");

        m_imgSize.Width = m_playerHeadTex.Width;
        m_imgSize.Height = m_playerHeadTex.Height;

        pos.X = 0;
        pos.Y = 0;
        pos.Z = 0;

        speed = 0;
        angle = 0;
        vSpeed = 0;

        health = 0;

        state = STATE_MOVING;
        return true;
      } catch {
        return false;
      }
    }

    public override bool draw(SpriteBatch spriteHandler, Rectangle? dest) {
      if (dest != null) {
        if (state == Player.STATE_MOVING && (angle<=10 && angle >=-10))
        {
            spriteHandler.Draw(m_playerHeadTex, (Rectangle)dest, null, Color.White);
        }
        else if ((state == Player.STATE_DYING) || (state == Player.STATE_FALLING))
        {
            spriteHandler.Draw(m_currentFallingFrame, (Rectangle)dest, null, Color.White);  
        }
        else if (state == Player.STATE_MOVING && (angle < -10 && angle > -30))
        {
            spriteHandler.Draw(m_playerHeadLeftTex, (Rectangle)dest, null, Color.White);
        }
        else if (state == Player.STATE_MOVING && (angle > 10 && angle < 30))
        {
            spriteHandler.Draw(m_playerHeadRightTex, (Rectangle)dest, null, Color.White);
        }
        else if (state == Player.STATE_MOVING && angle <= -30)
        {
            spriteHandler.Draw(m_playerHeadExtremeLeftTex, (Rectangle)dest, null, Color.White);
        }
        else if (state == Player.STATE_MOVING && angle >= 30)
        {
            spriteHandler.Draw(m_playerHeadExtremeRightTex, (Rectangle)dest, null, Color.White);
        }
        else if ((state==Player.STATE_GETTINGUP) || (state==Player.STATE_DEAD))
        {
          spriteHandler.Draw(m_playerStuckInSnow, (Rectangle)dest, null, Color.White);
        }
        return true;
      } else {
        return false;
      }
    }

    public override bool animate(GameTime gameTime) {
      if (m_state == Player.STATE_FALLING) {
        m_stateTimer += gameTime.ElapsedGameTime.Milliseconds;

        if (m_stateTimer >= FLYFORWARD_TIME_MILLIS) {
          m_currentFallingFrame = m_playerFallingTwo;
        }

        // if we've stopped moving, done with falling animation, and playing getting up animation
        if (((vSpeed == 0) && (speed == 0)) || (m_stateTimer > FALLING_STATE_TIMEOUT)) {
          return true;
        }
      
      }else if (m_state == Player.STATE_DYING) {
        m_stateTimer += gameTime.ElapsedGameTime.Milliseconds;

        if (m_stateTimer >= FLYFORWARD_TIME_MILLIS) {
          m_currentFallingFrame = m_playerFallingTwo;
        }

        // if we've stopped moving, done with falling animation, and playing getting up animation
        if (((vSpeed == 0) && (speed == 0)) || (m_stateTimer > FALLING_STATE_TIMEOUT)) {
          return true;
        }

      }else if (m_state == Player.STATE_GETTINGUP) {
        m_stateTimer += gameTime.ElapsedGameTime.Milliseconds;
        if (m_stateTimer >= GETTINGUP_TIME_MILLIS) {
          return true;
        }
      }
      return false; 
    }
  }
}
